package com.jeeplus.test.materialManagement.materialType.service;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.jeeplus.core.domain.BaseEntity;
import com.jeeplus.sys.service.dto.OfficeDTO;
import com.jeeplus.sys.service.dto.UserDTO;
import com.jeeplus.sys.utils.StringUtils;
import com.jeeplus.sys.utils.UserUtils;
import com.jeeplus.test.materialManagement.materialType.domain.MaterialTypeInfo;
import com.jeeplus.test.materialManagement.materialType.mapper.MaterialTypeMapper;
import org.apache.commons.compress.utils.Lists;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * @author: 王强
 * @create: 2022-11-25 09:13
 **/
@Service
public class MaterialTypeService {

    @Resource
    private MaterialTypeMapper mapper;

    public List<MaterialTypeInfo> list(MaterialTypeInfo info) {
        LambdaQueryWrapper<MaterialTypeInfo> wrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(info.getName())) {
            wrapper.like(MaterialTypeInfo::getName, info.getName());
        }
        if (info.getSort() != null) {
            wrapper.like(MaterialTypeInfo::getSort, info.getSort());
        }
//        wrapper.eq(CwReimbursementTypeInfo::getBusinessType, "1");

        wrapper.eq(BaseEntity::getDelFlag, 0);
        wrapper.orderByAsc(MaterialTypeInfo::getSort);
        return mapper.findList(wrapper);
    }


    /**
     * 根据id查询项目详情
     * @param id
     * @return
     */
    public MaterialTypeInfo getById(String id) {
        MaterialTypeInfo info = mapper.getById(id);
        return info;
    }

    public List<MaterialTypeInfo> bxList(MaterialTypeInfo info) {
        LambdaQueryWrapper<MaterialTypeInfo> wrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(info.getName())) {
            wrapper.like(MaterialTypeInfo::getName, info.getName());
        }
        if (info.getSort() != null) {
            wrapper.like(MaterialTypeInfo::getSort, info.getSort());
        }

        wrapper.eq(BaseEntity::getDelFlag, 0);
        wrapper.orderByAsc(MaterialTypeInfo::getSort);
        return mapper.selectList(wrapper);
    }

    public List<MaterialTypeInfo> cgList(MaterialTypeInfo info) {
        LambdaQueryWrapper<MaterialTypeInfo> wrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(info.getName())) {
            wrapper.like(MaterialTypeInfo::getName, info.getName());
        }
        if (info.getSort() != null) {
            wrapper.like(MaterialTypeInfo::getSort, info.getSort());
        }

        wrapper.eq(BaseEntity::getDelFlag, 0);
        wrapper.orderByAsc(MaterialTypeInfo::getSort);
        return mapper.selectList(wrapper);
    }

    public String save(MaterialTypeInfo info) {
        // parentId未传值，则表示一级菜单
        if(StringUtils.isEmpty(info.getParentId())) {
            info.setParentId("0");
        }
        // 保存数据
        if (StringUtils.isNotEmpty(info.getId())) {
            return update(info);
        }
        return add(info);
    }

    /**
     * 新增
     * @param info
     * @return
     */
    public String add(MaterialTypeInfo info) {
        // 判断名称是否已存在
        Integer isExist = mapper.checkNameIsExist(info.getName(), info.getParentId());
        if (isExist > 0) {
            return "false";
        }
        // 获取当前登录人信息
        UserDTO userDto = UserUtils.getCurrentUserDTO();
        // 生成id值
        String id = UUID.randomUUID().toString().replace("-", "");
        info.setId(id);
        info.setCreateBy(userDto.getId());
        info.setCreateDate(new Date());
        info.setUpdateBy(userDto.getId());
        info.setUpdateDate(new Date());
        info.setDelFlag(0);
        // 生成序号/层级
        getNo(info);
        mapper.insert(info);
        return "操作完成";
    }

    /**
     * 修改
     * @param info
     * @return
     */
    public String update(MaterialTypeInfo info) {
        // 获取当前登录人信息
        UserDTO userDto = UserUtils.getCurrentUserDTO();
        info.setUpdateBy(userDto.getId());
        info.setUpdateDate(new Date());
        mapper.updateById(info);
        return "操作完成";
    }

    /**
     * 生成序号/层级
     * @param info
     * @return
     */
    public MaterialTypeInfo getNo(MaterialTypeInfo info) {
        MaterialTypeInfo parentInfo = null;
        if (!"0".equals(info.getParentId())) {
            //根据parentId查询父节点信息
            LambdaQueryWrapper<MaterialTypeInfo> wrapper = new LambdaQueryWrapper<>();
            wrapper.eq(BaseEntity::getDelFlag, 0);
            wrapper.eq(BaseEntity::getId, info.getParentId());
            parentInfo = mapper.selectOne(wrapper);
        }
        //查询序号
        Integer no = mapper.getNo(info.getParentId());
        // 该层级下有数据，正常返回序号值
        if (no != null) {
            info.setSort(no);
        } else {
            // 父节点存在，根据父节点序号生成；不存在，则表示无上级层级，从1开始
            if (parentInfo != null) {
                String s = parentInfo.getSort() + "01";
                info.setSort(Integer.parseInt(s));
            } else {
                info.setSort(1);
            }
        }
        // 生成层级
        //查询层级
        Integer level = mapper.getLevel(info.getParentId());
        if (level != null) {
            info.setLevel(level.toString());
        } else {
            // 父节点存在，根据父节点level生成，不存在，则表示无上级层级，从1开始
            if (parentInfo != null) {
                Integer i = Integer.parseInt(parentInfo.getLevel())+1;
                info.setLevel(i.toString());
            } else {
                info.setLevel("1");
            }
        }
        return info;
    }

    public String remove(String id) {
        //如果层级是一级或者是二级，则应该将一级下的所有信息删除或二级下的所有信息删除，反之为三级则直接删除
        MaterialTypeInfo typeInfo = mapper.getById(id);
        if (typeInfo.getLevel().equals("1")){
            //三个层级的情况
            mapper.deleteLevel3(id);
            //删除二级的信息
            mapper.deleteLevel2(id);
            //删除一级的信息
            mapper.deleteById(id);
        } else if (typeInfo.getLevel().equals("2")) {
            //两个层级的情况
            mapper.deleteLevel2(id);
            //删除二级的信息
            mapper.deleteById(id);
        } else if (typeInfo.getLevel().equals("3")) {
            mapper.deleteById(id);
        }
        return "操作成功";
    }
}
